body {
    font-family: sans-serif;
    text-align: center;
}

a {
    color: #000;
}

canvas {
    display: block;
    margin: auto;
}

h1 {
    text-align: center;
}

table {
    width: 1px;
    border-collapse: collapse;
    margin: auto;
    text-align: center;
    border-right: 1px solid #ccc;
}

tbody {
    border-top: 3px solid #ccc;
}

tbody.collapsed {
    border-top: 0;
}

tbody .dummy-row {
    display: none;
}

tbody.collapsed tr {
    display: none;
}

tbody.collapsed .dummy-row {
    display: table-row;
}

.group-name {
    cursor: pointer;
    text-align: left;
    vertical-align: top;
    white-space: nowrap;
}

.group-name:before {
    background: linear-gradient(to bottom, #999999 0%, #666666 75%, #333333 100%) repeat scroll 0 0 rgba(0, 0, 0, 0);
    border: 1px solid #333;
    border-radius: 3px;
    display: inline-block;
    font-size: 0.75em;
    line-height: 0.75em;
    padding: 3px;
    width: 0.75em;
    height: 0.75em;
    text-align: center;
    content: '-';
}

.collapsed .group-name:before {
    content: '+';
}


.collapsed td {
    display: none;
}

td, th {
    padding: 5px;
    border: 1px solid #ccc;
}

td {
    text-align: left;
    white-space: nowrap;
}

.dummy-cell {
    border: 0;
}

.metric-up,
.metric-up a,
.color-override .metric-down,
.color-override .metric-down a {
    color: #911;
    background-color: #f88;
}

.metric-up:before {
    content: '▲';
    text-decoration: none;
}

.metric-down,
.metric-down a,
.color-override .metric-up,
.color-override .metric-up a {
    color: #070;
    background-color: #cfc;
}

.metric-down:before {
    content: '▼';
    text-decoration: none;
}

.metric-none, .metric-none a {
    color: #333;
    background-color: #ddd;
}

.date-picker {
    margin: 20px;
    text-align: center;
}

/** Graph specific styles */

#graph {
    margin-bottom: 40px;
    display: inline-block;
    width: 1000px;
    height: 600px;
}

.xAxis .tickLabel {
    margin-top: 40px;
    white-space: nowrap;
    -ms-transform:rotate(-90deg); /* IE 9 */
    -moz-transform:rotate(-90deg); /* Firefox */
    -webkit-transform:rotate(-90deg); /* Safari and Chrome */
    -o-transform:rotate(-90deg); /* Opera */
    transform: rotate(-90deg);
}

.changes-block {
    display: inline-block;
    vertical-align: top;
}


/** Commit specific styles */

.debt-stats-table {
    width: 1px;
}

.debt-stats-table tbody {
    text-align: left;
}

.layout-table,
.layout-table td,
.layout-table tbody,
.layout-table tr {
    border: 0;
}

.description {
    text-align: left;
}


/* vs.css: https://github.com/richleland/pygments-css */

.highlight .hll { background-color: #ffffcc }
.highlight .c { color: #008000 } /* Comment */
.highlight .err { border: 1px solid #FF0000 } /* Error */
.highlight .k { color: #0000ff } /* Keyword */
.highlight .cm { color: #008000 } /* Comment.Multiline */
.highlight .cp { color: #0000ff } /* Comment.Preproc */
.highlight .c1 { color: #008000 } /* Comment.Single */
.highlight .cs { color: #008000 } /* Comment.Special */
.highlight .ge { font-style: italic } /* Generic.Emph */
.highlight .gh { font-weight: bold } /* Generic.Heading */
.highlight .gp { font-weight: bold } /* Generic.Prompt */
.highlight .gs { font-weight: bold } /* Generic.Strong */
.highlight .gu { font-weight: bold } /* Generic.Subheading */
.highlight .kc { color: #0000ff } /* Keyword.Constant */
.highlight .kd { color: #0000ff } /* Keyword.Declaration */
.highlight .kn { color: #0000ff } /* Keyword.Namespace */
.highlight .kp { color: #0000ff } /* Keyword.Pseudo */
.highlight .kr { color: #0000ff } /* Keyword.Reserved */
.highlight .kt { color: #2b91af } /* Keyword.Type */
.highlight .s { color: #a31515 } /* Literal.String */
.highlight .nc { color: #2b91af } /* Name.Class */
.highlight .ow { color: #0000ff } /* Operator.Word */
.highlight .sb { color: #a31515 } /* Literal.String.Backtick */
.highlight .sc { color: #a31515 } /* Literal.String.Char */
.highlight .sd { color: #a31515 } /* Literal.String.Doc */
.highlight .s2 { color: #a31515 } /* Literal.String.Double */
.highlight .se { color: #a31515 } /* Literal.String.Escape */
.highlight .sh { color: #a31515 } /* Literal.String.Heredoc */
.highlight .si { color: #a31515 } /* Literal.String.Interpol */
.highlight .sx { color: #a31515 } /* Literal.String.Other */
.highlight .sr { color: #a31515 } /* Literal.String.Regex */
.highlight .s1 { color: #a31515 } /* Literal.String.Single */
.highlight .ss { color: #a31515 } /* Literal.String.Symbol */

// git diff highlights

.highlight .gi { color: #070; }
.highlight .gd { color: #911; }
